=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2006-12-11 22:44:38 +0000
+++ loggerhead/controllers/revision_ui.py	2006-12-12 00:47:38 +0000
@@ -69,6 +69,7 @@
             'children': children,
             'author': rev.committer,
             'comment': comment,
+            'comment_clean': [util.html_clean(s) for s in comment],
             'date': datetime.datetime.fromtimestamp(rev.timestamp),
             'changes': changes,
             'branch_name': turbogears.config.get('loggerhead.branch_name'),

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2006-12-11 21:51:02 +0000
+++ loggerhead/history.py	2006-12-12 00:47:38 +0000
@@ -187,6 +187,11 @@
             'revno': self.get_revno(parent_revid),
         } for parent_revid in rev.parent_ids]
 
+        if len(parents) == 0:
+            left_parent = None
+        else:
+            left_parent = rev.parent_ids[-1]
+            
         entry = {
             'revid': revid,
             'revno': self.get_revno(revid),
@@ -195,7 +200,9 @@
             'age': util.timespan(now - commit_time) + ' ago',
             'short_comment': short_comment,
             'comment': rev.message,
+            'comment_clean': util.html_clean(rev.message),
             'parents': [util.Container(p) for p in parents],
+            'changes': self.diff_revisions(revid, left_parent, get_diffs=False),
         }
         return util.Container(entry)
     
@@ -226,7 +233,7 @@
         else:
             yield ('>', None, None)
     
-    def diff_revisions(self, revid, otherrevid):
+    def diff_revisions(self, revid, otherrevid, get_diffs=True):
         """
         Return a nested data structure containing the changes between two
         revisions::
@@ -245,6 +252,8 @@
                     ),
                 ),
             )
+        
+        if C{get_diffs} is false, the C{chunks} will be omitted.
         """
 
         new_tree = self._branch.repository.revision_tree(revid)
@@ -269,20 +278,12 @@
             tree_file = bzrlib.textfile.text_file(tree.get_file(fid))
             return tree_file.readlines()
         
-        def fix_line(s):
-            # to avoid using <pre>, we need to turn spaces into nbsp, and kid
-            # doesn't notice u'\xa0', so we have to do it by hand, and escape
-            # html also.  oh well.
-            s = cgi.escape(s)
-            return s.replace(' ', '&nbsp;')
-        
         def process_diff(diff):
             chunks = []
             chunk = None
             for line in diff.splitlines():
-                if len(line) == 1:
-                    # i think this happens because bazaar gets too aggressive about removing trailing whitespace
-                    line = line + ' '
+                if len(line) == 0:
+                    continue
                 if line.startswith('+++ ') or line.startswith('--- '):
                     continue
                 if line.startswith('@@ '):
@@ -296,22 +297,44 @@
                     new_lineno = lines[1]
                 elif line.startswith('  '):
                     chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=new_lineno,
-                                                     type='context', line=fix_line(line[2:])))
+                                                     type='context', line=util.html_clean(line[2:])))
                     old_lineno += 1
                     new_lineno += 1
                 elif line.startswith('+ '):
                     chunk.diff.append(util.Container(old_lineno=None, new_lineno=new_lineno,
-                                                     type='insert', line=fix_line(line[2:])))
+                                                     type='insert', line=util.html_clean(line[2:])))
                     new_lineno += 1
                 elif line.startswith('- '):
                     chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=None,
-                                                     type='delete', line=fix_line(line[2:])))
+                                                     type='delete', line=util.html_clean(line[2:])))
+                    old_lineno += 1
+                elif line.startswith(' '):
+                    # why does this happen?
+                    chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=new_lineno,
+                                                     type='context', line=util.html_clean(line[1:])))
+                    old_lineno += 1
+                    new_lineno += 1
+                elif line.startswith('+'):
+                    # why does this happen?
+                    chunk.diff.append(util.Container(old_lineno=None, new_lineno=new_lineno,
+                                                     type='insert', line=util.html_clean(line[1:])))
+                    new_lineno += 1
+                elif line.startswith('-'):
+                    # why does this happen?
+                    chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=None,
+                                                     type='delete', line=util.html_clean(line[1:])))
                     old_lineno += 1
                 else:
-                    chunk.diff.append(util.Container(type='unknown', line=fix_line(repr(line))))
+                    chunk.diff.append(util.Container(old_lineno=None, new_lineno=None,
+                                                     type='unknown', line=util.html_clean(repr(line))))
+            if chunk is not None:
+                chunks.append(chunk)
             return chunks
                     
         def handle_modify(old_path, new_path, fid, kind):
+            if not get_diffs:
+                modified.append(util.Container(filename=rich_filename(new_path, kind)))
+                return
             old_lines = tree_lines(old_tree, fid)
             new_lines = tree_lines(new_tree, fid)
             buffer = StringIO()

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2006-12-11 22:44:38 +0000
+++ loggerhead/static/css/style.css	2006-12-12 00:47:38 +0000
@@ -21,7 +21,7 @@
     color: #708a96;
 	font-weight: normal;
 	font-size: 1.6em;
-    margin-bottom: 0.5em;
+    margin-bottom: 1.5em;
 }
 
 .branch-name {
@@ -149,65 +149,104 @@
 
 /* ----- changelog page ----- */
 
-.log-entries {
-    border: none;
-    font-size: smaller;
-    margin-bottom: 1em;
-    padding: 0;
-    background-color: #f4f5f8;
-    border: 1px solid #8cacbb;
-    width: 100%;
-}
-
 /* secret css-jutsu to make hidden margins vanish on a table */
 .log-entries table {
 	border-collapse: collapse;
-    width: 99.9%;
+    width: 100%;
+}
+
+.log-entries .revision-header {
+    background-color: #dee7ec;
+    padding: 3px;
+    width: 80%;
+	margin-top: 1em;
+	margin-left: 1em;
+}
+
+.log-entries .revision-header .revision-number {
+	width: 6em;
+	padding-left: 0.5em;
+}
+
+.log-entries .revision-log {
+	width: 60%;
+	border: none;
+	margin-left: 8em;
+	margin-bottom: 1.5em;
+}
+
+.log-entries .revision-log table {
+	font-size: smaller;
+}
+
+.log-entries .revision-log th {
+	width: 8em;
+	text-align: right;
+	padding-right: 0.5em;
+	vertical-align: top;
+}
+
+.log-entries .revision-log td.date {
+/*	font-family: monospace; */
+}
+
+.log-entries .revision-log td.files {
+	font-family: monospace;
+}
+
+.xxlog-entries {
+    border: none;
+    font-size: smaller;
+    margin-bottom: 1em;
+    padding: 0;
+    background-color: #f4f5f8;
+    border: 1px solid #8cacbb;
+    width: 100%;
 }
 
 /* left and right cols are fixed size */
-.log-entries table col.header,col.mark-diff {
+.xxlog-entries table col.header,col.mark-diff {
 	width: 100pt;
 }
 
-.log-entries table .header {
+.xxlog-entries table .header {
 	padding-left: 0.5em;
 	padding-right: 0.5em;
 	padding-top: 2px;
 	padding-bottom: 2px;
 }
 
-.log-entries table .data {
+.xxlog-entries table .data {
 	padding-left: 0.5em;
 	padding-top: 2px;
 	padding-bottom: 2px;
 }
 
-.log-entries table .top {
+.xxlog-entries table .top {
 	padding-top: 0.3em;
 }
 
-.log-entries table .date {
+.xxlog-entries table .date {
 	font-family: monospace;
 	padding-bottom: 0.3em;
 	border-bottom: solid 1px #8cacbb;
 }
 
-.log-entries th {
+.xxlog-entries th {
     background-color: #dee7ec;
     color: black;
 	font-weight: normal;
 	text-align: right;
 }
 
-.log-entries .firstline {
+.xxlog-entries .firstline {
     font-weight: bold;
     text-align: left;
     padding-top: 0.5em;
     padding-bottom: 0.5em;
 }
 
-.log-entries td {
+.xxlog-entries td {
     font-weight: normal;
     text-align: left;
 }
@@ -309,6 +348,9 @@
 .diff-chunk .delete {
 	background-color: #ffa0a0;
 }
+.diff-chunk td.text {
+	padding-left: 1em;
+}
 
 .diff .lineno {
 	color: purple;

=== modified file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2006-12-11 21:51:02 +0000
+++ loggerhead/templates/changelog.kid	2006-12-12 00:47:38 +0000
@@ -26,12 +26,58 @@
 </span>
 
 <div class="log-entries">
-    <table>
+    <div py:for="entry in changes" class="revision">
+        <div class="revision-header">
+            <table>
+                <tr>
+                    <td class="revision-number"> 
+                        <a title="Show revision" href="${tg.url([ '/revision', entry.revid ])}" class="revno"> ${entry.revno} </a>
+                    </td>
+					<td>
+					    <a title="Show revision" href="${tg.url([ '/revision', entry.revid ])}" class="revno"> ${entry.short_comment} </a>
+					</td>
+				</tr>
+			</table>
+        </div>
+        
+        <div class="revision-log">
+        <table>
+	        <tr>
+	            <th class="author">committer:</th>
+	            <td class="author"> ${util.hide_email(entry.author)} </td>
+	        </tr>
+	        <tr>
+	            <th class="date">date:</th>
+	            <td class="date"> ${entry.date.strftime('%d %b %Y %H:%M')} &nbsp; (${entry.age}) </td>
+	        </tr>
+	        <!--tr class="divider"> <th></th> <td></td> </tr-->
+	        <tr py:if="entry.changes.added">
+	            <th class="files"> files added: </th>
+	            <td class="files"> <span py:for="filename in entry.changes.added">${filename} <br /></span> </td>
+	        </tr>
+	        <tr py:if="entry.changes.removed">
+	            <th class="files"> files removed: </th>
+	            <td class="files"> <span py:for="filename in entry.changes.removed">${filename} <br /></span> </td>
+	        </tr>
+	        <tr py:if="entry.changes.renamed">
+	            <th class="files"> files renamed: </th>
+	            <td class="files"> <span py:for="old_filename, new_filename in entry.changes.renamed">${old_filename} => ${new_filename}<br /></span> </td>
+	        </tr>
+	        <tr py:if="entry.changes.modified">
+	            <th class="files"> files modified: </th>
+	            <td class="files"> <span py:for="item in entry.changes.modified">${item.filename} <br /></span> </td>
+	        </tr>
+        </table>
+	        </div>
+    </div>
+    
+    <!--table>
         <col class="header" />
         <col class="data" />
         <col class="mark-diff" />
         
-        <span py:for="entry in changes">
+        <div py:for="entry in changes">
+            
             <tr>
                 <th class="firstline header age"> ${entry.age} </th>
                 <th class="firstline data"> ${entry.short_comment} </th>
@@ -46,7 +92,6 @@
                 </span></td>
             </tr>
             
-            <!-- !for multiple-parents: -->
             <span py:if="len(entry.parents) > 1">
                 <span py:for="parent in entry.parents">
                     <tr>
@@ -65,8 +110,7 @@
                 <td class="date data"> ${entry.date.strftime('%d %b %Y %H:%M')} </td>
                 <td class="date mark-diff"> &nbsp; </td>
             </tr>
-        </span>
-    </table>
+        </span-->
 </div>
 
 </body>

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2006-12-11 21:51:02 +0000
+++ loggerhead/templates/revision.kid	2006-12-12 00:47:38 +0000
@@ -44,7 +44,7 @@
         </tr>
         <tr>
             <th class="description">description:</th>
-            <td class="description"> <span py:for="line in comment">${line} <br /></span> </td>
+            <td class="description"> <span py:for="line in comment_clean">${XML(line)} <br /></span> </td>
         </tr>
         <tr class="divider"> <th></th> <td></td> </tr>
         <tr py:if="changes.added">
@@ -75,7 +75,7 @@
                 <tr py:for="line in chunk.diff">
                     <td class="lineno">${line.old_lineno}</td>
                     <td class="lineno">${line.new_lineno}</td>
-                    <td class="${line.type}">${XML(line.line)}</td>
+                    <td class="${line.type} text">${XML(line.line)}</td>
                 </tr>
             </table>
         </td></tr>

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2006-12-11 21:51:02 +0000
+++ loggerhead/util.py	2006-12-12 00:47:38 +0000
@@ -16,6 +16,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import cgi
 import re
 import sha
 
@@ -133,3 +134,12 @@
         if pos - n >= 0:
             out.insert(0, -n)
 
+
+def html_clean(s):
+    """
+    clean up a string for html display.  expand any tabs, encode any html
+    entities, and replace spaces with '&nbsp;'.  this is primarily for use
+    in displaying monospace text.
+    """
+    s = cgi.escape(s.expandtabs()).replace(' ', '&nbsp;')
+    return s;

